@@ -2,7 +2,8 @@
 
 ;; Copyright (C) 1985, 1987, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
 ;;   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-;;   2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
+;;   2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019. 2020,
+;;   2021
 ;;   Free Software Foundation, Inc.
 
 ;; Authors:    2004- Alan Mackenzie
@@ -162,7 +163,7 @@ Works with: topmost-intro-cont."
       (c-safe-position (or containing-sexp (point)) c-state-cache)
       containing-sexp))))
 
-(defun c-lineup-arglist (langelem)
+(defun c-lineup-arglist (_langelem)
   "Line up the current argument line under the first argument.
 
 As a special case, if the indented line is inside a brace block
@@ -268,7 +269,7 @@ Works with: arglist-cont, arglist-cont-nonempty."
 	    (c-forward-syntactic-ws))
 	(c-lineup-argcont-scan other-match)))))
 
-(defun c-lineup-arglist-intro-after-paren (langelem)
+(defun c-lineup-arglist-intro-after-paren (_langelem)
   "Line up a line to just after the open paren of the surrounding paren
 or brace block.
 
@@ -325,7 +326,7 @@ if (  x < 10
 
 Since this function doesn't do anything for lines without an infix
 operator you typically want to use it together with some other line-up
-settings, e.g. as follows \(the arglist-close setting is just a
+settings, e.g. as follows (the arglist-close setting is just a
 suggestion to get a consistent style):
 
 \(c-set-offset 'arglist-cont '(c-lineup-arglist-operators 0))
@@ -486,7 +487,7 @@ Works with: func-decl-cont."
 	    (vector (+ (current-column) c-basic-offset)))
 	c-basic-offset))))
 
-(defun c-indent-one-line-block (langelem)
+(defun c-indent-one-line-block (_langelem)
   "Indent a one line block `c-basic-offset' extra.
 E.g.:
 
@@ -509,7 +510,7 @@ Work with: Almost all syntactic symbols, but most useful on *-open."
 	  c-basic-offset
 	nil))))
 
-(defun c-indent-multi-line-block (langelem)
+(defun c-indent-multi-line-block (_langelem)
   "Indent a multi line block `c-basic-offset' extra.
 E.g.:
 
@@ -645,7 +646,7 @@ Works with: The `c' syntactic symbol."
 		  (goto-char (c-langelem-pos langelem)))))
 	  (vector (current-column)))))))
 
-(defun c-lineup-comment (langelem)
+(defun c-lineup-comment (_langelem)
   "Line up a comment start according to `c-comment-only-line-offset'.
 If the comment is lined up with a comment starter on the previous
 line, that alignment is preserved.
@@ -670,7 +671,7 @@ Works with: comment-intro."
 	    -1000))			;jam it against the left side
        ))))
 
-(defun c-lineup-knr-region-comment (langelem)
+(defun c-lineup-knr-region-comment (_langelem)
   "Line up a comment in the \"K&R region\" with the declaration.
 That is the region between the function or class header and the
 beginning of the block.  E.g.:
@@ -793,6 +794,38 @@ arglist-cont-nonempty."
   (or (c-lineup-assignments langelem)
       c-basic-offset))
 
+(defun c-lineup-ternary-bodies (langelem)
+  "Line up true and false branches of a ternary operator (i.e. `?:').
+More precisely, if the line starts with a colon which is a part of
+a said operator, align it with corresponding question mark; otherwise
+return nil.  For example:
+
+    return arg % 2 == 0 ? arg / 2
+                        : (3 * arg + 1);    <- c-lineup-ternary-bodies
+
+Works with: arglist-cont, arglist-cont-nonempty and statement-cont."
+  (save-excursion
+    (back-to-indentation)
+    (when (and (eq ?: (char-after))
+               (not (eq ?: (char-after (1+ (point))))))
+      (let ((limit (c-langelem-pos langelem)) (depth 1))
+        (catch 'done
+          (while (and (c-syntactic-skip-backward "^?:" limit t)
+		      (not (bobp)))
+            (backward-char)
+            (cond ((eq (char-after) ??)
+                   ;; If we've found a question mark, decrease depth.  If we've
+                   ;; reached zero, we've found the one we were looking for.
+                   (when (zerop (setq depth (1- depth)))
+                     (throw 'done (vector (current-column)))))
+                  ((or (eq ?: (char-before)) (eq ?? (char-before)))
+                   ;; Step over `::' and `?:' operators.  We don't have to
+                   ;; handle `?:' here but doing so saves an iteration.
+                   (if (eq (point) limit)
+                       (throw 'done nil)
+                     (goto-char (1- (point)))))
+                  ((setq depth (1+ depth)))))))))) ; Otherwise increase depth.
+
 (defun c-lineup-cascaded-calls (langelem)
   "Line up \"cascaded calls\" under each other.
 If the line begins with \"->\" or \".\" and the preceding line ends
@@ -839,7 +872,7 @@ arglist-cont-nonempty."
 
 	  (vector col))))))
 
-(defun c-lineup-string-cont (langelem)
+(defun c-lineup-string-cont (_langelem)
   "Line up a continued string under the one it continues.
 A continued string in this sense is where a string literal follows
 directly after another one.  E.g.:
@@ -864,7 +897,7 @@ arglist-cont-nonempty."
 	     (goto-char pos)
 	     (vector (current-column)))))))
 
-(defun c-lineup-template-args (langelem)
+(defun c-lineup-template-args (_langelem)
   "Line up template argument lines under the first argument.
 To allow this function to be used in a list expression, nil is
 returned if there's no template argument on the first line.
@@ -994,7 +1027,7 @@ Works with: objc-method-args-cont."
 	    (+ curcol (- prev-col-column (current-column)))
 	  c-basic-offset)))))
 
-(defun c-lineup-inexpr-block (langelem)
+(defun c-lineup-inexpr-block (_langelem)
   "Line up the block for constructs that use a block inside an expression,
 e.g. anonymous classes in Java and lambda functions in Pike.  The body
 is aligned with the start of the header, e.g. with the \"new\" or
@@ -1022,7 +1055,7 @@ Works with: inlambda, inexpr-statement, inexpr-class."
 	(goto-char (cdr res))
 	(vector (current-column))))))
 
-(defun c-lineup-whitesmith-in-block (langelem)
+(defun c-lineup-whitesmith-in-block (_langelem)
   "Line up lines inside a block in Whitesmith style.
 It's done in a way that works both when the opening brace hangs and
 when it doesn't.  E.g.:
@@ -1077,7 +1110,7 @@ Works with: brace-list-entry, brace-entry-open, statement,
 arglist-cont."
   (save-excursion
     (goto-char (c-langelem-pos langelem))
-    (when (looking-at "\\s\(")
+    (when (looking-at "\\s(")
       (if (c-go-up-list-backward)
 	  (let ((pos (point)))
 	    (back-to-indentation)
@@ -1086,7 +1119,7 @@ arglist-cont."
 	      (vector (+ (current-column) c-basic-offset))))
 	(vector 0)))))
 
-(defun c-lineup-2nd-brace-entry-in-arglist (langelem)
+(defun c-lineup-2nd-brace-entry-in-arglist (_langelem)
   "Lineup the second entry of a brace block under the first, when the first
 line is also contained in an arglist or an enclosing brace ON THAT LINE.
 
@@ -1127,7 +1160,7 @@ Works with brace-list-intro."
 	      (eq (char-after) ?{))))
        'c-lineup-arglist-intro-after-paren))
 
-(defun c-lineup-class-decl-init-+ (langelem)
+(defun c-lineup-class-decl-init-+ (_langelem)
   "Line up the second entry of a class (etc.) initializer c-basic-offset
 characters in from the identifier when:
 \(i) The type is a class, struct, union, etc. (but not an enum);
@@ -1168,7 +1201,7 @@ Works with: brace-list-intro."
 	    (eq (point) init-pos)
 	    (vector (+ (current-column) c-basic-offset)))))))
 
-(defun c-lineup-class-decl-init-after-brace (langelem)
+(defun c-lineup-class-decl-init-after-brace (_langelem)
   "Line up the second entry of a class (etc.) initializer after its opening
 brace when:
 \(i) The type is a class, struct, union, etc. (but not an enum);
@@ -1210,7 +1243,7 @@ Works with: brace-list-intro."
 	    (eq (point) init-pos)
 	    'c-lineup-arglist-intro-after-paren)))))
 
-(defun c-lineup-cpp-define (langelem)
+(defun c-lineup-cpp-define (_langelem)
   "Line up macro continuation lines according to the indentation of
 the construct preceding the macro.  E.g.:
 
@@ -1249,7 +1282,7 @@ That is useful in a list expression to specify the default indentation
 on the top level.
 
 If `c-syntactic-indentation-in-macros' is nil then this function keeps
-the current indentation, except for empty lines \(ignoring the ending
+the current indentation, except for empty lines (ignoring the ending
 backslash) where it takes the indentation from the closest preceding
 nonempty line in the macro.  If there's no such line in the macro then
 the indentation is taken from the construct preceding it, as described
@@ -1359,7 +1392,7 @@ Works with: Any syntactic symbol which has an anchor position."
     (vector (current-column))))
     
 
-(defun c-lineup-dont-change (langelem)
+(defun c-lineup-dont-change (_langelem)
   "Do not change the indentation of the current line.
 
 Works with: Any syntactic symbol."
@@ -1367,7 +1400,7 @@ Works with: Any syntactic symbol."
     (back-to-indentation)
     (vector (current-column))))
 
-(defun c-lineup-respect-col-0 (langelem)
+(defun c-lineup-respect-col-0 (_langelem)
   "If the current line starts at column 0, return [0].  Otherwise return nil.
 
 This can be used for comments (in conjunction with, say,
@@ -1380,7 +1413,7 @@ anchored there, but reindent other comments."
       nil)))
 
 
-(defun c-snug-do-while (syntax pos)
+(defun c-snug-do-while (syntax _pos)
   "Dynamically calculate brace hanginess for do-while statements.
 Using this function, `while' clauses that end a `do-while' block will
 remain on the same line as the brace that closes that block.
@@ -1398,7 +1431,7 @@ ACTION associated with `block-close' syntax."
 	  '(before)
 	'(before after)))))
 
-(defun c-snug-1line-defun-close (syntax pos)
+(defun c-snug-1line-defun-close (_syntax pos)
   "Determine the brace hanginess for an AWK defun-close.
 If the action/function being closed is a one-liner, keep it so.  Otherwise put
 the closing brace on its own line."
