@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 1988, 1994, 1996, 2000, 2001, 2002, 2003, 2004, 2005,
 ;;   2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-;;   2017, 2018, 2019
+;;   2017, 2018, 2019, 2020, 2021
 ;;   Free Software Foundation, Inc.
 
 ;; Author: Alan Mackenzie <acm@muc.de> (originally based on awk-mode.el)
@@ -51,9 +51,11 @@
     (load "cc-bytecomp" nil t)))
 
 (cc-require 'cc-defs)
+(cc-require-when-compile 'cc-langs)
+(cc-require-when-compile 'cc-fonts)
+(cc-require 'cc-engine)
 
 ;; Silence the byte compiler.
-(cc-bytecomp-defvar font-lock-mode)	; Checked with boundp before use.
 (cc-bytecomp-defvar c-new-BEG)
 (cc-bytecomp-defvar c-new-END)
 
@@ -96,7 +98,7 @@
 ;; Emacs has in the past used \r to mark hidden lines in some fashion (and
 ;; maybe still does).
 
-(defconst c-awk-esc-pair-re "\\\\\\(.\\|\n\\|\r\\|\\'\\)")
+(defconst c-awk-esc-pair-re "\\\\\\(.\\|\n\\|\\'\\)")
 ;;   Matches any escaped (with \) character-pair, including an escaped newline.
 (defconst c-awk-non-eol-esc-pair-re "\\\\\\(.\\|\\'\\)")
 ;;   Matches any escaped (with \) character-pair, apart from an escaped newline.
@@ -218,10 +220,10 @@
 (defconst c-awk-neutrals*-re
   (concat "\\(" c-awk-neutral-re "\\)*"))
 ;;   A (possibly empty) string of neutral characters (or character pairs).
-(defconst c-awk-var-num-ket-re "[]\)0-9a-zA-Z_$.\x80-\xff]+")
+(defconst c-awk-var-num-ket-re "[])0-9a-zA-Z_$.]+")
 ;;   Matches a char which is a constituent of a variable or number, or a ket
-;; (i.e. closing bracKET), round or square.  Assume that all characters \x80 to
-;; \xff are "letters".
+;; (i.e. closing bracKET), round or square.  (2019-07): No longer assume that
+;; characters \x80 to \xff are "letters".
 (defconst c-awk-div-sign-re
   (concat c-awk-var-num-ket-re c-awk-neutrals*-re "/"))
 ;;   Will match a piece of AWK buffer ending in / which is a division sign, in
@@ -230,7 +232,7 @@
 ;; will only work when there won't be a preceding " or / before the sought /
 ;; to foul things up.
 (defconst c-awk-non-arith-op-bra-re
-  "[[\({&=:!><,?;'~|]")
+  "[[({&=:!><,?;'~|]")
 ;;   Matches an opening BRAcket (of any sort), or any operator character
 ;; apart from +,-,/,*,%.  For the purpose at hand (detecting a / which is a
 ;; regexp bracket) these arith ops are unnecessary and a pain, because of "++"
@@ -269,7 +271,7 @@
 ;; Matches optional whitespace followed by a "/" with string syntax (a matched
 ;; regexp delimiter).
 (defconst c-awk-space*-unclosed-regexp-/-re
-  (concat c-awk-escaped-nls*-with-space* "\\s\|"))
+  (concat c-awk-escaped-nls*-with-space* "\\s|"))
 ;; Matches optional whitespace followed by a "/" with string fence syntax (an
 ;; unmatched regexp delimiter).
 
@@ -595,7 +597,7 @@
   ;; starts at a `while' token.
   (not (c-get-char-property (c-point 'eol) 'c-awk-NL-prop)))
 
-(defun c-awk-clear-NL-props (beg end)
+(defun c-awk-clear-NL-props (beg _end)
   ;; This function is run from before-change-hooks.  It clears the
   ;; c-awk-NL-prop text property from beg to the end of the buffer (The END
   ;; parameter is ignored).  This ensures that the indentation engine will
@@ -652,6 +654,46 @@
 ;; several lines back.  The elisp "advice" feature is used on these functions
 ;; to allow this.
 
+(defun c-awk-font-lock-invalid-namespace-separators (limit)
+  ;; This function will be called from font-lock for a region bounded by POINT
+  ;; and LIMIT, as though it were to identify a keyword for
+  ;; font-lock-keyword-face.  It always returns NIL to inhibit this and
+  ;; prevent a repeat invocation.  See elisp/lispref page "Search-based
+  ;; Fontification".
+  ;;
+  ;; This function gives invalid GAWK namepace separators (::)
+  ;; font-lock-warning-face.  "Invalid" here means there are spaces, etc.,
+  ;; around a separator, or there are more than one of them in an identifier.
+  ;; Invalid separators inside function declaration parentheses are handled
+  ;; elsewhere.
+  (while (and
+	  (< (point) limit)
+	  (c-syntactic-re-search-forward
+	   (eval-when-compile
+	     (concat "\\([^" (c-lang-const c-symbol-chars awk) "]::\\)"
+		     "\\|"
+		     ;; "\\(::[^" (c-lang-const c-symbol-start awk) "]\\)"
+		     "\\(::[^" c-alpha "_" "]\\)"
+		     "\\|"
+		     "\\(::[" (c-lang-const c-symbol-chars awk) "]*::\\)"))
+	   limit 'bound))
+    (cond
+     ((match-beginning 1) 		; " ::"
+      (c-put-font-lock-face (1+ (match-beginning 1)) (match-end 1)
+			    'font-lock-warning-face)
+      (goto-char (- (match-end 1) 2)))
+     ((match-beginning 2)		; ":: "
+      (c-put-font-lock-face (match-beginning 2) (1- (match-end 2))
+			    'font-lock-warning-face)
+      (goto-char (1- (match-end 2))))
+     (t					; "::foo::"
+      (c-put-font-lock-face (match-beginning 3) (+ 2 (match-beginning 3))
+			    'font-lock-warning-face)
+      (c-put-font-lock-face (- (match-end 3) 2) (match-end 3)
+			    'font-lock-warning-face)
+      (goto-char (- (match-end 3) 2)))))
+    nil)
+
 (defun c-awk-beginning-of-logical-line (&optional pos)
 ;; Go back to the start of the (apparent) current line (or the start of the
 ;; line containing POS), returning the buffer position of that point.  I.e.,
@@ -850,7 +892,7 @@
 ;; Just beyond logical line following the region which is about to be changed.
 ;; Set in c-awk-record-region-clear-NL and used in c-awk-after-change.
 
-(defun c-awk-record-region-clear-NL (beg end)
+(defun c-awk-record-region-clear-NL (_beg end)
 ;; This function is called exclusively from the before-change-functions hook.
 ;; It does two things: Finds the end of the (logical) line on which END lies,
 ;; and clears c-awk-NL-prop text properties from this point onwards.  BEG is
@@ -903,6 +945,13 @@
     (goto-char c-new-BEG)
     (c-awk-set-syntax-table-properties c-new-END)))
 
+(defun c-awk-context-expand-fl-region (beg end)
+  ;; Return a cons (NEW-BEG . NEW-END), where NEW-BEG is the beginning of the
+  ;; logical line BEG is on, and NEW-END is the beginning of the line after
+  ;; the end of the logical line that END is on.
+  (cons (save-excursion (c-awk-beginning-of-logical-line beg))
+	(c-awk-beyond-logical-line end)))
+
 ;; Awk regexps written with help from Peter Galbraith
 ;; <galbraith@mixing.qc.dfo.ca>.
 ;; Take GNU Emacs's 'words out of the following regexp-opts.  They don't work
@@ -910,18 +959,34 @@
 (defconst awk-font-lock-keywords
   (eval-when-compile
     (list
-     ;; Function names.
-     '("^\\s *\\(func\\(tion\\)?\\)\\>\\s *\\(\\sw+\\)?"
-       (1 font-lock-keyword-face) (3 font-lock-function-name-face nil t))
-     ;;
+     ;; Function declarations.
+     `(,(c-make-font-lock-search-function
+	 "^\\s *\\(func\\(tion\\)?\\)\\s +\\(\\(\\sw+\\(::\\sw+\\)?\\)\\s *\\)?\\(([^()]*)\\)?"
+	 '(1 font-lock-keyword-face t)
+	 ;; We can't use LAXMATCH in `c-make-font-lock-search-function', so....
+	 '((when (match-beginning 4)
+	     (c-put-font-lock-face
+	      (match-beginning 4) (match-end 4) font-lock-function-name-face)
+	     nil))
+	 ;; Put warning face on any use of :: inside the parens.
+	 '((when (match-beginning 6)
+	     (goto-char (1+ (match-beginning 6)))
+	     (let ((end (1- (match-end 6))))
+	       (while (and (< (point) end)
+			   (c-syntactic-re-search-forward "::" end t))
+		 (c-put-font-lock-face (- (point) 2) (point)
+				       'font-lock-warning-face)))
+	     nil))))
+
      ;; Variable names.
      (cons
       (concat "\\<"
 	      (regexp-opt
 	       '("ARGC" "ARGIND" "ARGV" "BINMODE" "CONVFMT" "ENVIRON"
-		 "ERRNO" "FIELDWIDTHS" "FILENAME" "FNR" "FS" "IGNORECASE"
-		 "LINT" "NF" "NR" "OFMT" "OFS" "ORS" "PROCINFO" "RLENGTH"
-		 "RS" "RSTART" "RT" "SUBSEP" "TEXTDOMAIN") t) "\\>")
+		 "ERRNO" "FIELDWIDTHS" "FILENAME" "FNR" "FPAT" "FS" "FUNCTAB"
+		 "IGNORECASE" "LINT" "NF" "NR" "OFMT" "OFS" "ORS" "PREC"
+		 "PROCINFO" "RLENGTH" "ROUNDMODE" "RS" "RSTART" "RT" "SUBSEP"
+		 "SYNTAB" "TEXTDOMAIN") t) "\\>")
       'font-lock-variable-name-face)
 
      ;; Special file names.  (acm, 2002/7/22)
@@ -952,7 +1017,8 @@ std\\(err\\|in\\|out\\)\\|user\\)\\)\\>\
      ;; Keywords.
      (concat "\\<"
 	     (regexp-opt
-	      '("BEGIN" "END" "break" "case" "continue" "default" "delete"
+	      '("BEGIN" "BEGINFILE" "END" "ENDFILE"
+		"break" "case" "continue" "default" "delete"
 		"do" "else" "exit" "for" "getline" "if" "in" "next"
 		"nextfile" "return" "switch" "while")
 	      t) "\\>")
@@ -962,16 +1028,20 @@ std\\(err\\|in\\|out\\)\\|user\\)\\)\\>\
 	       ,(concat
 		 "\\<"
 		 (regexp-opt
-		  '("adump" "and" "asort" "atan2" "bindtextdomain" "close"
-		    "compl" "cos" "dcgettext" "exp" "extension" "fflush"
-		    "gensub" "gsub" "index" "int" "length" "log" "lshift"
-		    "match" "mktime" "or" "print" "printf" "rand" "rshift"
+		  '("adump" "and" "asort" "asorti" "atan2" "bindtextdomain" "close"
+		    "compl" "cos" "dcgettext" "dcngettext" "exp" "extension" "fflush"
+		    "gensub" "gsub" "index" "int" "isarray" "length" "log" "lshift"
+		    "match" "mktime" "or" "patsplit" "print" "printf" "rand" "rshift"
 		    "sin" "split" "sprintf" "sqrt" "srand" "stopme"
 		    "strftime" "strtonum" "sub" "substr"  "system"
-		    "systime" "tolower" "toupper" "xor") t)
+		    "systime" "tolower" "toupper" "typeof" "xor")
+		  t)
 		 "\\>")
 	       0 c-preprocessor-face-name))
 
+     ;; Directives
+     `(eval . '("@\\(include\\|load\\|namespace\\)\\>" 0 ,c-preprocessor-face-name))
+
      ;; gawk debugging keywords.  (acm, 2002/7/21)
      ;; (Removed, 2003/6/6.  These functions are now fontified as built-ins)
      ;;	(list (concat "\\<" (regexp-opt '("adump" "stopme") t) "\\>")
@@ -983,6 +1053,9 @@ std\\(err\\|in\\|out\\)\\|user\\)\\)\\>\
 		c-awk-escaped-nls*-with-space* "(")
        (0 'font-lock-warning-face))
 
+     ;; Double :: tokens, or the same with space(s) around them.
+     #'c-awk-font-lock-invalid-namespace-separators
+
      ;; Space after \ in what looks like an escaped newline.  2002/5/31
      '("\\\\\\s +$" 0 font-lock-warning-face t)
 
